% SETUP_FLEXPEN_CONFIGURATION
%
% SETUP_FLEXPEN_CONFIGURATION calculates and returns the model parameters
% of the flexible pendulum Quanser plant.
%
% Flexible pendulum system nomenclature:
% m_link            Mass of flexible link                           (kg)
% m_link_s          Mass of flexible link + strain gage             (kg)
% m_weight          Mass of the weight at the end of the flexible link 
%                   w/ four screws and bolts.                       (kg)
% m_connector       Mass of connector that fastens the STIFFPEN and 
%                   the FLEXPEN together                            (kg)
% m_flexpen         Total mass of flexible pendulum -- FLEXPEN      (kg)
% P_SG              Position on flexible link where strain gage 
%                   measures                                        (m)
% L_link            Length from P_SG to tip of link                 (m)
% l_cm_link         Length from FLEXPEN pivot to CM of link         (m)
% l_cm_link_s       Length from FLEXPEN pivot to CM of link with
%                   strain gage attached                            (m)
% L_sw              Length from FLEXPEN pivot to start of weight    (m)
% L_weight          Length of FLEXPEN weight                        (m)
% l_cm_weight       Length from FLEXPEN pivot to CM of FLEXPEN weight (m)
% J_flexpen_pivot   Moment of inertia of a flexible pendulum
%                   about end point.                                (kg.m^2)
% l_flexpen_cm      Center-of-mass of FLEXPEN when considered a
%                   single rigid body.                              (m)
% J_flexpen_cm      FLEXPEN moment of inertia about its CM.         (kg.m^2)
% B_flexpen         Viscous damping of flexible pendulum        (N.m.s/rad)
% fn                Natural frequency of FLEXPEN                    (Hz)
% wn                Natural frequency of FLEXPEN                    (rad/s)
% Ks                Flexible link stiffness with weight         (N.m/rad)
% K_SG              Strain gage deflection                          (in/V)
% K_GAGE            Sensitivity of strain gage sensor
% (rad/in)
%
% Copyright (C) 2005 Quanser Consulting Inc.
% Quanser Consulting Inc.
%
%
function [ m_flexpen, L_link, l_cm_link, J_flexpen_cm, B_flexpen, Ks, K_GAGE ] = setup_flexpen_configuration ()
%
% Conversion factors:
% from Inch to Meter
K_IN2M = 0.0254;
% from Meter to Inch
K_M2IN = 1/0.0254;
%
% Mass of flexible link (kg)
m_link = 0.065;
% Mass of flexible link + strain gage (kg)
m_link_s = 0.157;
% Mass of the weight at the end of the flexible link w/ four screws and bolts (kg).
m_weight = 0.256;
% Mass of connector that fastens the STIFFPEN and the FLEXPEN together (kg).
m_connector = 0.0450;
% Total mass of flexible pendulum -- FLEXPEN
m_flexpen = m_connector + m_link_s + m_weight;
%
% Position on flexible link where strain gage begins to measure (m).
P_SG = 3.25 * K_IN2M;
% Distance between P_SG and pivot point of FLEXPEN (i.e. PS_G to the middle
% of the thumbscrew holes on the strain gage module) (m).
L_sg_pivot = (1 + 5/8) * K_IN2M;
% Length from P_SG to tip of link, i.e. the flexible length that is detectable (m)
L_link = 15.5 * K_IN2M;
% Length from FLEXPEN pivot to CM of link (m)
l_cm_link = 5.75 * K_IN2M + L_sg_pivot;
% Length from FLEXPEN pivot to CM of link with strain gage attached (m)
l_cm_link_s = 1.125 * K_IN2M  + L_sg_pivot;
% Length from FLEXPEN pivot to start of weight (m)
L_sw = 12.50 * K_IN2M + L_sg_pivot;
% Length of FLEXPEN weight (m)
L_weight = 3.0 * K_IN2M;
% Length from FLEXPEN pivot to CM of FLEXPEN weight (m)
l_cm_weight = L_sw + L_weight/2;
%
% Moment of inertia of FLEXPEN about end point (kg.m^2).
% The FLEXPEN is composed of two bodeies: the flexible link and the weight
% at the end pendulum.
% mi = mass of body i
% Li = length of body i
M1 = m_link;
M2 = m_weight;
L1 = L_sw;
L2 = L_weight;
J_flexpen_pivot = 1/3*M1*L1^2 + M2*L1^2 + M2*L1*L2 + 1/3*M2*L2^2;
% Center-of-mass of FLEXPEN when considered a single rigid body (m).
l_flexpen_cm = ( m_link_s * l_cm_link_s + m_weight * l_cm_weight ) / ( m_link_s  + m_weight );
% FLEXPEN moment of inertia about its CM (kg.m^2).
J_flexpen_cm = ( m_link_s  + m_weight ) * l_flexpen_cm / 12;
% Assume viscous damping of flexible beam is negligible (N.m.s/rad).
B_flexpen = 0;
%
% Natural frequency of FLEXPEN experimentally determined (Hz)
fn = 1.25;
% Natural frequency experimentally determined (rad/s)
wn = 2 * pi * fn;
% Flexible link stiffness with weight determined experimentally. (N.m/rad)
Ks = wn^2 * J_flexpen_pivot;
% Strain gage deflection (in/V)
K_SG = 1; 
% Sensitivity of strain gage sensor (rad/in)
K_GAGE = K_SG / ( L_link * K_M2IN );

