%% SETUP_IP02_FLEXPEN
%
% IP02 Flexible Inverted Pendulum (FLEXPEN) Control Lab: 
% Design of a LQR position controller to balance rigid and flexible
% pendulum.
% 
% SETUP_IP02_FLEXPEN sets the IP02 and FLEXPEN system's 
% model parameters accordingly to the user-defined configuration.
% SETUP_FLEXPEN can also set the controllers' parameters, 
% accordingly to the user-defined desired specifications.
%
% Copyright (C) 2012 Quanser Consulting Inc.
% Quanser Consulting Inc.
%
clear
%
%% User-defined Configuration

% Type of Cart Load: set to 'NO_LOAD', 'WEIGHT'
LOAD_TYPE = 'NO_WEIGHT';
% LOAD_TYPE = 'WEIGHT';
% Type of single pendulum: set to 'LONG_24IN', 'MEDIUM_12IN'
PEND_TYPE = 'LONG_24IN'; 
%PEND_TYPE = 'MEDIUM_12IN'; 
% Turn on or off the safety watchdog on the cart position: set it to 1 , or 0 
X_LIM_ENABLE = 1;       % safety watchdog turned ON
%X_LIM_ENABLE = 0;      % safety watchdog turned OFF
% Safety Limits on the cart displacement (m)
X_MAX = 0.30;            % cart displacement maximum safety position (m)
X_MIN = - X_MAX;        % cart displacement minimum safety position (m)
% Turn on or off the safety watchdog on the pendulum angle: set it to 1 , or 0 
ALPHA_LIM_ENABLE = 1;       % safety watchdog turned ON
%ALPHA_LIM_ENABLE = 0;      % safety watchdog turned OFF
% Safety Limits on the pendulum angle (deg)
global ALPHA_MAX ALPHA_MIN
ALPHA_MAX = 20;            % pendulum angle maximum safety position (deg)
ALPHA_MIN = - ALPHA_MAX;   % pendulum angle minimum safety position (deg)
% Amplifier Gain used: set to VoltPAQ to 1
K_AMP = 1;
% Amplifier Type: set to 'VoltPAQ' or 'Q3'
AMP_TYPE = 'VoltPAQ';
% AMP_TYPE = 'Q3';
% Digital-to-Analog Maximum Voltage (V); for MultiQ cards set to 10
VMAX_DAC = 10;
%
%
%% Controller Configuration and Filter Settings
% Type of Controller: set it to 'LQR_AUTO', 'LQR_GUI_TUNING', 'MANUAL'  
CONTROLLER_TYPE = 'LQR_AUTO';    % LQR controller design: automatic mode
%CONTROLLER_TYPE = 'MANUAL';    % controller design: manual mode
% Initial Condition on alpha, i.e. pendulum angle at t = 0 (deg)
%IC_ALPHA0 = 0.1;
IC_ALPHA0 = 0;
% conversion to radians
IC_ALPHA0 = IC_ALPHA0 / 180 * pi;
% Initialization of Simulink diagram parameters
    % Cart Encoder Resolution
    global K_EC K_EP
    % IP02 CART second-order filter parameters
    wcf_c = 2 * pi * 35; % filter cutting frequency    
    zetaf_c = 0.9;        % filter damping ratio
    % STIFF PENDULUM second-order filter parameters
    wcf_sp = 2 * pi * 30; % filter cutting frequency    
    zetaf_sp = 0.9;        % filter damping ratio
    % FLEXIBLE PENDULUM second-order filter parameters
    % wcf_fp = 2 * pi * 20; % filter cutting frequency 
    wcf_fp = 2 * pi * 4; % filter cutting frequency 
    zetaf_fp = 0.9;        % filter damping ratio
%
%% Load Model
% variables required in the Simulink diagrams
global VMAX_AMP IMAX_AMP

% Set the model parameters accordingly to the user-defined IP01 or IP02 system configuration.
% These parameters are used for model representation and controller design.
[ Rm, Jm, Kt, eta_m, Km, Kg, eta_g, Mc, r_mp, Beq ] = config_ip02( LOAD_TYPE, AMP_TYPE );

% Lumped Mass of the Cart System (accounting for the rotor inertia)
% Mc = Mc + Eff_g * Kg^2 * Jm / r_mp^2;

% Set the model parameters for the single pendulum accordingly to the user-defined system configuration.
[ g, Msp, Lsp, lsp, Jsp, Bsp ] = config_sp( PEND_TYPE );

% Set the model parameters for the single pendulum accordingly to the user-defined system configuration.
[ Mfp, Lfp, lfp, Jfp, Bfp, Ks, K_GAGE  ] = config_flexpen();
%
% For the following state vector: X = [ xc; alpha; gamma; xc_dot; alpha_dot; gamma_dot ]
% Initialization of the State-Space Representation of the Open-Loop System
% Call the following Maple-generated file to initialize the State-Space Matrices: A, B, C, and D
FLEXPEN_ABCD_eqns;
% Add actuator dynamics
%[ An, Bn ] = d_flexpen_actuator_dynamics( A, B, AMP_TYPE, Rm, Kt, Eff_m, Km, Kg, Eff_g, Mc, r_mp );
%
%% LQR Control Design
% Set to 'LQR_AUTO' or 'MANUAL'.
if strcmp ( CONTROLLER_TYPE, 'LQR_AUTO' )
    % Augment (A,B) to include cart integrator
    A(7,1) = 1;
    A(7,7) = 0;
    B(7) = 0;
    C = eye(3,7);
    
    % Set LQR weighting matrices.
        Q = diag( [ 50 0 0 10 0 0 0.5] );
        R = 1;

    % Automatically calculate the LQR controller gain
    K = lqr( A, B, Q, R );
    % Display the calculated gains    
    disp( ' ' )
    disp( 'Calculated LQR controller gain elements: ' )
    if strcmp( AMP_TYPE, 'Q3')
        disp( [ 'K(1) = ' num2str( K(1) ) ' A/m' ] )
        disp( [ 'K(2) = ' num2str( K(2) ) ' A/rad' ] )
        disp( [ 'K(3) = ' num2str( K(3) ) ' A/rad' ] )
        disp( [ 'K(4) = ' num2str( K(4) ) ' A.s/m' ] )
        disp( [ 'K(5) = ' num2str( K(5) ) ' A.s/rad' ] )
        disp( [ 'K(6) = ' num2str( K(6) ) ' A.s/rad' ] )
        disp( [ 'K(7) = ' num2str( K(7) ) ' A/(m.s)' ] )
    else        
        disp( [ 'K(1) = ' num2str( K(1) ) ' V/m' ] )
        disp( [ 'K(2) = ' num2str( K(2) ) ' V/rad' ] )
        disp( [ 'K(3) = ' num2str( K(3) ) ' V/rad' ] )
        disp( [ 'K(4) = ' num2str( K(4) ) ' V.s/m' ] )
        disp( [ 'K(5) = ' num2str( K(5) ) ' V.s/rad' ] )
        disp( [ 'K(6) = ' num2str( K(6) ) ' V.s/rad' ] )
        disp( [ 'K(7) = ' num2str( K(7) ) ' V/(m.s)' ] )
    end
elseif strcmp ( CONTROLLER_TYPE, 'MANUAL' )
    K = [ 0 0 0 0 0 0];
    disp( ' ' )
    disp( 'STATUS: manual mode' ) 
    disp( 'The model parameters of your IP01/IP02+SIP+FLEXPEN system have been set.' )
    disp( 'You can now design your state-feedback position controller.' )
    disp( ' ' )
else
    error( 'Error: Please set the type of controller that you wish to implement.' )
end